// dfa.h cDFAi񌈒萫LI[g}gj̃f
#ifndef __MERCURY_DFA__
#define __MERCURY_DFA__

#include <map>
#include <set>

namespace mercury
{
	// f[^^
	typedef uintptr_t         state_t;     // 
	typedef std::set<state_t> state_set_t; // Ԃ̏W

	const state_t STATE_ERROR = static_cast<state_t>(-1);


	template<typename _Input>
	class dfa
	{
	public:
		typedef std::pair<state_t, _Input>            state_input_t;
		typedef std::map<state_input_t, state_t>      transition_t;
		typedef typename transition_t::const_iterator transition_const_iterator_t;

	public:
		dfa(const state_t &start)
		{
			m_start = start;
		}
		dfa(const state_t &start, const state_set_t &accepts, const transition_t &transition)
		{
			m_start      = start;
			m_accepts    = accepts;
			m_transition = transition;
		}

		// J
		state_t transit(const state_t &state, const _Input &input) const
		{
			// ԂƓ͂̃^v쐬
			const state_input_t tuple(state, input);
			transition_const_iterator_t p = m_transition.find(tuple);
			if(p == m_transition.end())
			{
				// Jڐ悪Ȃ΃G[Ԃ
				return STATE_ERROR;
			}
			return p->second;
		}

		// Jڂ쐬
		void connect(const state_t &from, const _Input &input, const state_t &to)
		{
			// ԂƓ͂̃^v쐬
			const state_input_t tuple(from, input);
			m_transition[tuple] = to;
		}

		// Jڂ}[W
		void merge_transition(const dfa &rhs)
		{
			m_transition.insert(rhs.m_transition.begin(), rhs.m_transition.end());
		}

		// Ԑŏ
		bool compact(void)
		{
			// 
			return false;
		}

		// JnԂ̎擾/ݒ
		state_t get_start(void) const           { return m_start; }
		void    set_start(const state_t &start) { m_start = start; }

		// eԂ̎擾/ݒ
		state_set_t get_accepts(void) const                 { return m_accepts; }
		void        set_accepts(const state_set_t &accepts) { m_accepts = accepts; }
		void        add_accept (const state_t     &accept ) { m_accepts.insert(accept); }

		// eԂH
		bool is_accept_state(const state_t &state) const { return (m_accepts.find(state) != m_accepts.end()); }

		// ͂󗝉\H
		template<typename _ConstIterator>
		bool can_accept(_ConstIterator text_begin, _ConstIterator text_end) const
		{
			state_t state = get_start();
			for(_ConstIterator p = text_begin; p != text_end; p++)
			{
				state = transit(state, *p);
				if(state == STATE_ERROR) { return false; }
			}
			return is_accept_state(state);
		}

	private:
		state_t      m_start;                   // Jn
		state_set_t  m_accepts;                 // eԂ̏W
		transition_t m_transition;              // Jڃ}bv
	};
}

#endif // __MERCURY_DFA__
